@@ -46,6 +46,8 @@ struct obj {
 	size_t buffer_size;
 	struct ring ring;
 	int (*header_func)(struct obj *self, GstBuffer *buf);
+	uint64_t next_timestamp;
+	int bps;
 };
 
 struct obj_class {
@@ -150,12 +152,22 @@ vorbis_header(struct obj *self, GstBuffer *buf)
 	return 1;
 }
 
+static int
+flac_header(struct obj *self, GstBuffer *buf)
+{
+	unsigned type = (buf->data[0] & 0x7F);
+	/* frame sync code */
+	if (type == 127)
+		return 0;
+	return 1;
+}
+
 static inline void
 calculate_timestamp(struct obj *self, GstBuffer *out_buf)
 {
 	uint32_t samples;
 
-	samples = out_buf->size / (self->av_ctx->channels * sizeof(int16_t));
+	samples = out_buf->size / (self->av_ctx->channels * (self->bps >> 3));
 
 	out_buf->timestamp = self->timestamp;
 	out_buf->duration = gst_util_uint64_scale_int(samples, GST_SECOND, self->av_ctx->sample_rate);
@@ -168,34 +180,55 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 {
 	struct obj *self;
 	GstFlowReturn ret = GST_FLOW_OK;
+	AVCodecContext *av_ctx;
 
 	self = (struct obj *)((GstObject *)pad)->parent;
+	av_ctx = self->av_ctx;
 
 	if (G_UNLIKELY(!self->got_header)) {
 		int hdr = self->header_func(self, buf);
 		if (!hdr) {
+			GstCaps *new_caps;
+			GstStructure *s;
+			int bps;
+
 			self->got_header = true;
-			if (avcodec_open(self->av_ctx, self->codec) < 0) {
+			if (gst_av_codec_open(av_ctx, self->codec) < 0) {
 				ret = GST_FLOW_ERROR;
 				goto leave;
 			}
 
-			{
-				GstCaps *new_caps;
+#ifdef FF_API_GET_BITS_PER_SAMPLE_FMT
+			bps = av_get_bits_per_sample_fmt(av_ctx->sample_fmt);
+#else
+			bps = av_get_bytes_per_sample(av_ctx->sample_fmt) << 3;
+#endif
+			self->bps = bps;
 
-				new_caps = gst_caps_new_simple("audio/x-raw-int",
-						"rate", G_TYPE_INT, self->av_ctx->sample_rate,
+			s = gst_structure_new("audio/x-raw-int",
+					"rate", G_TYPE_INT, av_ctx->sample_rate,
+					"channels", G_TYPE_INT, av_ctx->channels,
+					"endianness", G_TYPE_INT, G_BYTE_ORDER,
+					"width", G_TYPE_INT, bps,
+					NULL);
+
+			switch (av_ctx->sample_fmt) {
+			case AV_SAMPLE_FMT_S16:
+			case AV_SAMPLE_FMT_S32:
+				gst_structure_set(s,
 						"signed", G_TYPE_BOOLEAN, TRUE,
-						"channels", G_TYPE_INT, self->av_ctx->channels,
-						"endianness", G_TYPE_INT, G_BYTE_ORDER,
-						"width", G_TYPE_INT, 16,
-						"depth", G_TYPE_INT, 16,
+						"depth", G_TYPE_INT, bps,
 						NULL);
-
-				GST_INFO_OBJECT(self, "caps are: %" GST_PTR_FORMAT, new_caps);
-				gst_pad_set_caps(self->srcpad, new_caps);
-				gst_caps_unref(new_caps);
+				break;
+			default:
+				break;
 			}
+
+			new_caps = gst_caps_new_full(s, NULL);
+
+			GST_INFO_OBJECT(self, "caps are: %" GST_PTR_FORMAT, new_caps);
+			gst_pad_set_caps(self->srcpad, new_caps);
+			gst_caps_unref(new_caps);
 		}
 	}
 
@@ -209,19 +242,29 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 		memcpy(pkt.data, buf->data, buf->size);
 		memset(pkt.data + pkt.size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
 
-		if (G_UNLIKELY(self->timestamp == GST_CLOCK_TIME_NONE))
-			self->timestamp = buf->timestamp;
+		if (G_UNLIKELY(self->timestamp == GST_CLOCK_TIME_NONE)) {
+			self->next_timestamp = self->timestamp = buf->timestamp;
+		} else if (self->next_timestamp != buf->timestamp) {
+			int64_t adj = self->next_timestamp - self->timestamp;
+
+			GST_WARNING_OBJECT(self, "reseting timestamp");
+			self->next_timestamp = self->timestamp = buf->timestamp;
+			self->timestamp -= adj;
+		}
+
+		self->next_timestamp += buf->duration;
 
 		do {
 			void *buffer_data;
 			int buffer_size;
 			int read;
+			unsigned total_buffer_size;
 
 			buffer_data = self->buffer_data + self->ring.in;
 			buffer_size = self->buffer_size - self->ring.in;
-			read = avcodec_decode_audio3(self->av_ctx, buffer_data, &buffer_size, &pkt);
+			read = avcodec_decode_audio3(av_ctx, buffer_data, &buffer_size, &pkt);
 			if (read < 0) {
-				ret = GST_FLOW_ERROR;
+				GST_WARNING_OBJECT(self, "error: %i", read);
 				break;
 			}
 
@@ -234,9 +277,14 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 				self->ring.out = 0;
 			}
 
-			if (self->ring.in - self->ring.out >= BUFFER_SIZE) {
+			if (BUFFER_SIZE > 0)
+				total_buffer_size = BUFFER_SIZE;
+			else
+				total_buffer_size = buffer_size;
+
+			if (self->ring.in - self->ring.out >= total_buffer_size) {
 				GstBuffer *out_buf;
-				out_buf = gst_buffer_new_and_alloc(BUFFER_SIZE);
+				out_buf = gst_buffer_new_and_alloc(total_buffer_size);
 				memcpy(out_buf->data, self->buffer_data + self->ring.out, out_buf->size);
 				calculate_timestamp(self, out_buf);
 				gst_buffer_set_caps(out_buf, self->srcpad->caps);
@@ -285,7 +333,7 @@ change_state(GstElement *element, GstStateChange transition)
 	switch (transition) {
 	case GST_STATE_CHANGE_READY_TO_NULL:
 		if (self->av_ctx) {
-			avcodec_close(self->av_ctx);
+			gst_av_codec_close(self->av_ctx);
 			av_freep(&self->av_ctx);
 		}
 		av_free_packet(&self->pkt);
@@ -312,11 +360,30 @@ sink_event(GstPad *pad, GstEvent *event)
 		ret = gst_pad_push_event(self->srcpad, event);
 		self->timestamp = GST_CLOCK_TIME_NONE;
 		break;
+	case GST_EVENT_EOS: {
+		/* flush current buffer */
+		GstBuffer *out_buf;
+		GstFlowReturn r;
+
+		out_buf = gst_buffer_new_and_alloc(self->ring.in - self->ring.out);
+		memcpy(out_buf->data, self->buffer_data + self->ring.out, out_buf->size);
+		calculate_timestamp(self, out_buf);
+		gst_buffer_set_caps(out_buf, self->srcpad->caps);
+
+		self->ring.out += out_buf->size;
+
+		r = gst_pad_push(self->srcpad, out_buf);
+		if (r != GST_FLOW_OK)
+			goto leave;
+		ret = gst_pad_push_event(self->srcpad, event);
+		break;
+	}
 	default:
 		ret = gst_pad_push_event(self->srcpad, event);
 		break;
 	}
 
+leave:
 	gst_object_unref(self);
 
 	return ret;
@@ -345,10 +412,11 @@ sink_setcaps(GstPad *pad, GstCaps *caps)
 		GstBuffer *buf;
 
 		codec_id = CODEC_ID_FLAC;
+		self->header_func = flac_header;
 
 		stream_header = gst_structure_get_value(in_struc, "streamheader");
 		if (!stream_header)
-			return false;
+			return true;
 
 		stream_info = gst_value_array_get_value(stream_header, 0);
 
@@ -384,8 +452,8 @@ generate_src_template(void)
 			"rate", GST_TYPE_INT_RANGE, 8000, 96000,
 			"signed", G_TYPE_BOOLEAN, TRUE,
 			"endianness", G_TYPE_INT, G_BYTE_ORDER,
-			"width", G_TYPE_INT, 16,
-			"depth", G_TYPE_INT, 16,
+			"width", GST_TYPE_INT_RANGE, 16, 32,
+			"depth", GST_TYPE_INT_RANGE, 16, 32,
 			"channels", GST_TYPE_INT_RANGE, 1, 256,
 			NULL);
 
