@@ -17,17 +17,13 @@
 #include <string.h> /* for memcpy */
 #include <stdbool.h>
 
+#include "gstav_parse.h"
+
 #define GST_CAT_DEFAULT gstav_debug
 
 static GstElementClass *parent_class;
 
-struct obj {
-	GstElement element;
-	GstPad *sinkpad, *srcpad;
-	AVCodec *codec;
-	AVCodecContext *av_ctx;
-	bool initialized;
-};
+#define obj gst_av_vdec
 
 struct obj_class {
 	GstElementClass parent_class;
@@ -45,8 +41,6 @@ static GstBuffer *convert_frame(struct obj *self, AVFrame *frame)
 	out_buf = gst_buffer_new_and_alloc(ctx->width * ctx->height * 3 / 2);
 	gst_buffer_set_caps(out_buf, self->srcpad->caps);
 
-	for (i = 0; i < ctx->height * ctx->width * 3 / 2; i++)
-		out_buf->data[i] = 0;
 	p = out_buf->data;
 	for (i = 0; i < ctx->height; i++)
 		memcpy(p + i * ctx->width, frame->data[0] + i * frame->linesize[0], ctx->width);
@@ -79,11 +73,14 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 		GstStructure *struc;
 
 		self->initialized = true;
-		if (avcodec_open(ctx, self->codec) < 0) {
+		if (gst_av_codec_open(ctx, self->codec) < 0) {
 			ret = GST_FLOW_ERROR;
 			goto leave;
 		}
 
+		if (self->parse_func)
+			self->parse_func(self, buf);
+
 		new_caps = gst_caps_new_empty();
 
 		struc = gst_structure_new("video/x-raw-yuv",
@@ -119,7 +116,7 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 
 	read = avcodec_decode_video2(ctx, frame, &got_pic, &pkt);
 	if (read < 0) {
-		ret = GST_FLOW_ERROR;
+		GST_WARNING_OBJECT(self, "error: %i", read);
 		goto leave;
 	}
 
@@ -163,7 +160,7 @@ change_state(GstElement *element, GstStateChange transition)
 	switch (transition) {
 	case GST_STATE_CHANGE_READY_TO_NULL:
 		if (self->av_ctx) {
-			avcodec_close(self->av_ctx);
+			gst_av_codec_close(self->av_ctx);
 			av_freep(&self->av_ctx);
 		}
 		break;
@@ -204,17 +201,6 @@ sink_setcaps(GstPad *pad, GstCaps *caps)
 	if (!ctx->time_base.num)
 		ctx->time_base = (AVRational){ 1, 0 };
 
-	codec_data = gst_structure_get_value(in_struc, "codec_data");
-	if (!codec_data)
-		goto next;
-	buf = gst_value_get_buffer(codec_data);
-	if (!buf)
-		goto next;
-	ctx->extradata = malloc(buf->size + FF_INPUT_BUFFER_PADDING_SIZE);
-	memcpy(ctx->extradata, buf->data, buf->size);
-	ctx->extradata_size = buf->size;
-
-next:
 	name = gst_structure_get_name(in_struc);
 	if (strcmp(name, "video/x-h263") == 0)
 		codec_id = CODEC_ID_H263;
@@ -294,6 +280,32 @@ next:
 	if (!self->codec)
 		return false;
 
+	switch (codec_id) {
+	case CODEC_ID_H263:
+		self->parse_func = gst_av_h263_parse;
+		break;
+	case CODEC_ID_H264:
+		self->parse_func = gst_av_h264_parse;
+		break;
+	case CODEC_ID_MPEG4:
+		self->parse_func = gst_av_mpeg4_parse;
+		break;
+	}
+
+	codec_data = gst_structure_get_value(in_struc, "codec_data");
+	if (!codec_data)
+		goto next;
+	buf = gst_value_get_buffer(codec_data);
+	if (!buf)
+		goto next;
+	ctx->extradata = malloc(buf->size + FF_INPUT_BUFFER_PADDING_SIZE);
+	memcpy(ctx->extradata, buf->data, buf->size);
+	ctx->extradata_size = buf->size;
+
+	if (self->parse_func)
+		self->parse_func(self, buf);
+
+next:
 	return true;
 }
 
