@@ -11,11 +11,37 @@
 
 #include "gstav_adec.h"
 #include "gstav_vdec.h"
+#include "gstav_h263enc.h"
+#include "gstav_h264enc.h"
 
 #include <stdbool.h>
 
 GstDebugCategory *gstav_debug;
 
+static GStaticMutex gst_av_codec_mutex = G_STATIC_MUTEX_INIT;
+
+int gst_av_codec_open(AVCodecContext *avctx, AVCodec *codec)
+{
+	int ret;
+
+	g_static_mutex_lock (&gst_av_codec_mutex);
+	ret = avcodec_open (avctx, codec);
+	g_static_mutex_unlock (&gst_av_codec_mutex);
+
+	return ret;
+}
+
+int gst_av_codec_close(AVCodecContext *avctx)
+{
+	int ret;
+
+	g_static_mutex_lock (&gst_av_codec_mutex);
+	ret = avcodec_close (avctx);
+	g_static_mutex_unlock (&gst_av_codec_mutex);
+
+	return ret;
+}
+
 static gboolean
 plugin_init(GstPlugin *plugin)
 {
